# -*- coding: utf-8 -*-
"""
Created on Fri Dec 28 17:36:34 2018

Utility functions
"""

########################################
### Functions


def write_log(text, file):
    f = open(file, 'a')
    f.write("{}\n".format(text))



# -*- coding: utf-8 -*-
import xarray as xr


def select_bounds(ms_nc, min_lat, max_lat, min_lon, max_lon):
    """
    Function to select out a bounding box of lat of lon (and to remove negative lon).

    Parameters
    ----------
    ms_nc : str
        Path to the MetService netcdf file.
    min_lat : int, float, or None
        The minimum lat to extract in WGS84 decimal degrees.
    max_lat : int, float, or None
        The maximum lat to extract in WGS84 decimal degrees.
    min_lon : int, float, or None
        The minimum lon to extract in WGS84 decimal degrees.
    max_lon : int, float, or None
        The maximum lon to extract in WGS84 decimal degrees.

    Returns
    -------
    xarray Dataset
        With the same data structure and attributes as the original netcdf
    """
    ms1 = xr.open_dataset(ms_nc)
    ms2 = ms1.where((ms1.longitude >= min_lon) & (ms1.longitude <= max_lon) & (ms1.latitude >= min_lat) & (ms1.latitude <= max_lat), drop=True)
    ms2 = ms2.where(ms1.longitude > 0, drop=True)
    ms1.close()

    return ms2


def to_df(ms_nc, to_rate=False):
    """
    Function to take a MetService netcdf file and convert it to a pandas DataFrame. Optionally to convert the precipitation values to an hourly rate.

    Parameters
    ----------
    ms_nc : str or xarray dataset
        Path to the MetService netcdf file or an xarray dataset object.
    to_rate : bool
        Should the accumulated precipitation be converted to an hourly rate?

    Returns
    -------
    DataFrame
    """
    if isinstance(ms_nc, xr.Dataset):
        ms1 = ms_nc
    elif isinstance(ms_nc, str):
        ms1 = xr.open_dataset(ms_nc)
    df1 = ms1.to_dataframe()
    ms1.close()
    df1.reset_index(level=['south_north', 'west_east'], drop=True, inplace=True)
    df1 = df1[df1.longitude > 0].reset_index()
    df1.time = df1.time.dt.round('H').values
    df1.rename(columns={'precipitation_amount': 'precip_accum'}, inplace=True)
    rem1 = df1.time.unique()[0]
    if to_rate:
        rem2 = df1.time.unique()[-1]
        df1a = df1[df1.time != rem2].copy()
        df1 = df1[df1.time != rem1]
        df1['other'] = df1a.precip_accum.values
        df1['precip_rate'] = df1['precip_accum'] - df1['other']
        df1.drop(['precip_accum', 'other'], axis=1, inplace=True)
    else:
        df1 = df1[df1.time != rem1]

    df1.set_index(['time', 'latitude', 'longitude'], inplace=True)

    return df1




# -*- coding: utf-8 -*-
"""
Created on Tue Oct  9 11:22:22 2018

@author: michaelek
"""
import pandas as pd
from nzmetservice import select_bounds, to_df, datasets

pd.options.display.max_columns = 10

#####################################
### Parameters

min_lat = -47
max_lat = -40
min_lon = 166
max_lon = 175
nc1_path = datasets.get_path('wrf_hourly_precip_nz8km_test')

####################################
### Import

ms1 = select_bounds(nc1_path, min_lat, max_lat, min_lon, max_lon)

ms_df = to_df(ms1, True).dropna().reset_index()
ms_df1 = ms_df[(ms_df.time <= '2018-09-24 12:00') & (ms_df.time >= '2018-09-24')]



























































# -*- coding: utf-8 -*-
import os
from re import search, IGNORECASE
from ftplib import FTP


def ftp_transfer(ftp_path, username, password, output_path):
    """
    Function to update the output_path with new netcdf forecast files from the MetService ftp site.

    Parameters
    ----------
    ftp_path : str
        The MetService ftp path.
    username : str
        The username to access the ftp site.
    password : str
        The password to access the ftp site.
    output_path : str
        The path where the netcdf files should be saved.

    Returns
    -------
    None
    """
    #############################################
    ### Get list of files in the output ftp dir

    files1 = [filename for filename in os.listdir(output_path) if search('.nc$', filename, IGNORECASE)]

    #############################################
    ### Get list of files in ftp site and compare to ours

    print('Transfer files from FTP site')
    ftp = FTP(ftp_path)
    ftp.login(user=username, passwd=password)
    ftp.cwd('netcdf_combined')
    ftp_files1 = ftp.mlsd()
    ftp_files2 = [i[0] for i in ftp_files1]
    ftp_files3 = [i for i in ftp_files2 if ('wrf_hourly_precip' in i) or ('NZ_WRF_Selection' in i)]
    ftp_files4 = [i for i in ftp_files3 if i not in files1]

    ##############################################
    ### Download files

    if ftp_files4:
        for f in ftp_files4:
            ftp.retrbinary('RETR ' + f, open(os.path.join(output_path, f), 'wb').write)
        print(str(len(ftp_files4)) + ' files downloaded')
    else:
        print('No files downloaded')



